// $Revision: 1$
// $Date: 6/18/2007 12:14:13 PM$
// $Author: Donnie Tognazzini$
// $NoKeywords$


/******************************************************************************
  CH_AsyncXMLPOSTRequest C'tor

    Parameters:
        [in] onSuccessCallBack
        [in] onFailureCallBack
        [in] responseMessageType

******************************************************************************/
function CH_AsyncXMLPOSTRequest( onSuccessCallBack, onFailureCallBack, responseMessageType )
{
    try
    {
        function onSuccessCallBackWrapper( responseMessage )
        {
            try
            {
                onSuccessCallBack( responseMessage );
            }
            catch ( e )
            {
                EX_Log( 'CH_AsyncXMLPOSTRequest::onSuccessCallBackWrapper( )\n' + e.message );
            }
        }

        function onFailureCallBackWrapper( errorMessage )
        {
            try
            {
                onFailureCallBack( errorMessage );
            }
            catch ( e )
            {
                EX_Log( 'CH_AsyncXMLPOSTRequest::onFailureCallBackWrapper( )\n' + e.message );
            }
        }

        var httpClient = new HttpClient( );

        httpClient.onError = function( errorMessage )
        {
            onFailureCallBackWrapper( errorMessage );
        };

        httpClient.onSuccess = function( xmlResponse )
        {
            try
            {
                var responseMessage = new responseMessageType( xmlResponse );

                onSuccessCallBackWrapper( responseMessage );
            }
            catch ( e )
            {
                onFailureCallBackWrapper( e.message );
            }
        };

        this.IsRequestInProgress = function( )
        {
            return httpClient.IsRequestInProgress( );
        };

        this.Stop = function( )
        {
            httpClient.reset( );
        }

        this.Start = function( primaryURL, secondaryURL, requestMessage )
        {
            try
            {
                httpClient.sendRequest( HTTP_VERB_POST,
                                        primaryURL,
                                        secondaryURL,
                                        "text/xml",
                                        requestMessage.asXML( ) );
            }
            catch ( e )
            {
                EX_ASSERT_NO_EXCEPTIONS( e, 'CH_AsyncXMLPOSTRequest::SendRequest( )' );
            }
        };
    }
    catch ( e )
    {
        EX_ASSERT_NO_EXCEPTIONS( e, 'CH_AsyncXMLPOSTRequest::CH_AsyncXMLPOSTRequest( )' );
    }
}

/******************************************************************************
  CH_AsyncRequestorBase C'tor

    Parameters:
        [in] onSuccess
        [in] onFailure
        [in] responseMessageType

******************************************************************************/
function CH_AsyncRequestorBase( onSuccess, onFailure, responseMessageType )
{
    try
    {
        this.m_asyncRequest = new CH_AsyncXMLPOSTRequest( onSuccess, onFailure, responseMessageType );

        this.Stop = function( )
        {
            try
            {
                this.m_asyncRequest.Stop( );
            }
            catch ( e )
            {
                EX_ASSERT_NO_EXCEPTIONS( e, 'CH_AsyncRequestorBase::Stop( )' );
            }
        };

        this.IsRequestInProgress = function( )
        {
            return this.m_asyncRequest.IsRequestInProgress( );
        };
    }
    catch ( e )
    {
        EX_ASSERT_NO_EXCEPTIONS( e, 'CH_AsyncRequestorBase::CH_AsyncRequestorBase( )' );
    }
}
