/******************************************************************************
    CallWave VoiceMail Widget

    Copyright (C) 2006 CallWave, Inc.  All Rights Reserved.

 ******************************************************************************/

/******************************************************************************
  SMS_WidgetViewController class
 ******************************************************************************/

/******************************************************************************
  SMS_WidgetViewController C'tor

 ******************************************************************************/
function SMS_WidgetViewController( onAccountInformationChanged )
{
    try
    {
        this.m_state            = this.VIEW_STATE_UNDEFINED;

        // Non-persistent account attributes
        this.m_name             = null;
        this.m_emailAddress     = null;
        this.m_phoneNumber      = null;

        this.m_onAccountInformationChanged = onAccountInformationChanged;

        this.m_downloadTracker = new DownloadTracker( );

        var self = this;

        this.m_loginAndRegisterControlSynchronizer = new MPC_MobileNumberControlSynchronizer( );

        /******************************
            Create the register view
        ******************************/
        var onRegisterAccountSuccess = function( responseMessage )
        {
            self.OnRegisterAccountSuccess( );
        };

        this.m_registerView = new SMS_RegisterView( this.m_loginAndRegisterControlSynchronizer, onRegisterAccountSuccess );

        /******************************
            Create the account handler
        ******************************/
        this.m_accountHandler = new ACC_AccountHandler( VM_WIDGET_SOURCE,
                                                        VM_WIDGET_VERSION,
                                                        VM_WIDGET_OS_VERSION );

         /******************************
            Create the auto-login view
        ******************************/
        var onGetAccountInfoSucceeded = function( responseMessage )
        {
            self.OnGetAccountInfoSucceeded( responseMessage );
        };

        var onLoginRequired = function( statusMessage, color )
        {
            self.OnLoginRequired( statusMessage, color );
        };

        this.m_autoLoginView = new SMS_AutoLoginView( this.m_accountHandler, onGetAccountInfoSucceeded, onLoginRequired );

        /******************************
            Create the login view
        ******************************/
        var onLoginSucceeded = function( responseMessage )
        {
            self.OnLoginSucceeded( responseMessage );
        };

        this.m_loginView = new SMS_LoginView( this.m_accountHandler, this.m_loginAndRegisterControlSynchronizer, onLoginSucceeded, onLoginRequired );

        /******************************
            Create the verification view
        ******************************/
        var onVerificationSuccess = function( )
        {
            self.OnVerificationSuccess( );
        };

        this.m_verificationView = new SMS_VerificationView( onVerificationSuccess, onLoginRequired );

        /******************************
            Create the SMS view
        ******************************/
        this.m_smsView = new SMS_SMSView( onLoginRequired );

        /******************************
            Initialize
        ******************************/

        this.m_loginAndRegisterControlSynchronizer.Initialize( );

        var uString = VM_GetPreference( PREF_KEY_U_STRING );
        var account = VM_GetPreference( PREF_KEY_ACCOUNT );

        if ( account !== null && account.length != 0 &&
             uString !== null && uString.length != 0 )
        {
            this.ShowAutoLoginView( );
            this.m_autoLoginView.StartAutoLogin( );
        }
        else if ( VM_GetPreference( PREF_KEY_SENDER_MOBILE ) !== null && ! this.GetForceRegistrationView( ) )
        {
            this.ShowLoginView( );
        }
        else
        {
            this.ShowRegistrationView( );
        }
    }
    catch ( e )
    {
        EX_ASSERT_NO_EXCEPTIONS( e, "SMS_WidgetViewController::SMS_WidgetViewController( )" );
    }
}
SMS_WidgetViewController.prototype =
{
    GetForceRegistrationView : function( )
    {
        try
        {
            var downRevPreference = VM_GetPreference( PREF_KEY_FORCE_REGISTRATION_START_SCREEN_PREFERENCE );

            return downRevPreference !== null && downRevPreference !== '';
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_WidgetViewController::GetForceRegistrationView( )" );
        }
    },

    ClearForceRegistrationView : function( )
    {
        try
        {
            // clear the down-rev key we're using to determine
            // if the user should be presented with the registration screen
            VM_SetPreference( PREF_KEY_FORCE_REGISTRATION_START_SCREEN_PREFERENCE, '' );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_WidgetViewController::ClearForceRegistrationView( )" );
        }
    },

    /**************************************************************************
      View State enum
     **************************************************************************/

    VIEW_STATE_UNDEFINED            : 0,
    VIEW_STATE_SHOW_REGISTER        : 1,
    VIEW_STATE_SHOW_AUTO_LOGIN      : 2,
    VIEW_STATE_SHOW_LOGIN           : 3,
    VIEW_STATE_SHOW_VERIFICATION    : 4,
    VIEW_STATE_SHOW_SMS             : 5,

    /**************************************************************************
      Accessors
     **************************************************************************/
    resetAccountInfo:function( )
    {
        this.m_name             = null;
        this.m_emailAddress     = null;
        this.m_phoneNumber      = null;
        this.m_phoneCC          = null;
        this.m_phoneNN          = null;

        VM_SetPreference( PREF_KEY_ACCOUNT,  '' );
        VM_SetPreference( PREF_KEY_U_STRING, '' );
    },

    GetLoginView:function( )
    {
        return this.m_loginView;
    },

    GetSMSView : function( )
    {
        return this.m_smsView;
    },

    GetRegisterView : function( )
    {
        return this.m_registerView;
    },

    GetVerificationView : function( )
    {
        return this.m_verificationView;
    },

    UpdatePreferences : function( maxCalls, showActivation )
    {
        try
        {
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'SMS_WidgetViewController::UpdatePreferences( )' );
        }
    },

    /**************************************************************************
      Event handler methods
     **************************************************************************/

    handleRegistrationSubmit:function( )
    {
        this.m_registerView.handleRegistrationSubmit( );
    },

    handleAuthenticateSubmit:function( )
    {
        this.m_loginView.handleAuthenticateSubmit( );
    },

    handlePinReminderRequested:function( )
    {
        this.m_loginView.handlePinReminderRequested( );
    },

    handleVerifySubmit : function( )
    {
        this.m_verificationView.HandleVerifySubmit( );
    },

    handleVerifyPhone : function( )
    {
        this.m_verificationView.VerifyPhone( '' );
    },

    /**************************************************************************
      State management methods
     **************************************************************************/

    getState:function( )
    {
        return this.m_state;
    },

    getName: function()
    {
        return this.m_name;
    },

    getPhoneNumber: function()
    {
        return this.m_phoneNumber;
    },

    getEmailAddress: function()
    {
        return this.m_emailAddress;
    },

    OnRegisterAccountSuccess:function( )
    {
        this.ClearForceRegistrationView( );

        this.m_autoLoginView.handleGetAccountInfo( );
    },

    Logout : function( )
    {
        this.m_loginView.OnLogout( );

        this.ShowLoginView( );

        this.resetAccountInfo( );
    },

    OnVerificationStartOver : function( )
    {
        this.m_loginView.OnLogout( );

        this.ShowRegistrationView( );

        this.resetAccountInfo( );
    },

    CancelAutoLogin: function()
    {
        this.m_autoLoginView.StopRequest( );
    },

    ShowAutoLoginView: function( )
    {
        this.ChangeState( this.VIEW_STATE_SHOW_AUTO_LOGIN );
    },

    ShowLoginView: function( )
    {
        this.ChangeState( this.VIEW_STATE_SHOW_LOGIN );
    },

    ShowRegistrationView: function( )
    {
        this.ChangeState( this.VIEW_STATE_SHOW_REGISTER );
    },

    ChangeState: function( newState )
    {
        if ( newState == this.m_state )
        {
            return;
        }

        this.m_state = newState;
        this.updateView( );
    },

    OnLoginRequired:function( statusMessage, color )
    {
        try
        {
            this.ChangeState( this.VIEW_STATE_SHOW_LOGIN );

            if ( statusMessage !== undefined )
            {
                this.m_loginView.SetStatusText( statusMessage, color );
            }
        }
        catch ( e )
        {
            EX_Log( "SMS_WidgetViewController::OnLoginRequired( ) : " + e.message );
        }
    },

    NotifyAccountInformationChanged: function( )
    {
        try
        {
            if ( this.m_onAccountInformationChanged )
            {
                this.m_onAccountInformationChanged( this.m_name, this.m_emailAddress, this.m_phoneNumber );
            }
        }
        catch ( e )
        {
            EX_Log( "SMS_WidgetViewController::NotifyAccountInformationChanged( ) : " + e.message );
        }
    },

    OnGetAccountInfoSucceeded:function( responseMessage )
    {
        try
        {
            // copy over all the attributes
            this.m_name         = responseMessage.m_name;
            this.m_emailAddress = responseMessage.m_email;
            this.m_phoneNumber  = responseMessage.m_phone;
            this.m_phoneCC      = responseMessage.m_phoneCC;
            this.m_phoneNN      = responseMessage.m_phoneNN;

            VM_SetPreference( PREF_KEY_SENDER_EMAIL, this.m_emailAddress );
            VM_SetPreference( PREF_KEY_SENDER_NAME,  this.m_name );

            this.NotifyAccountInformationChanged( );

            this.m_verificationView.OnLoginSucceeded( responseMessage.m_carrierId,
                                                      responseMessage.m_hideRegCarrier,
                                                      this.m_phoneNumber,
                                                      this.m_phoneCC,
                                                      this.m_phoneNN );

            this.m_smsView.OnLoginSucceeded( responseMessage.m_smsBodyOH,
                                             responseMessage.m_maxSMSParts,
                                             responseMessage.m_showCarriers,
                                             responseMessage.m_okButtonDisplayDelayMS );

            if ( responseMessage.m_phoneVerified == 0 )
            {
                this.ChangeState( this.VIEW_STATE_SHOW_VERIFICATION );
                this.m_verificationView.VerifyPhone( '' );
            }
            else
            {
                this.ChangeState( this.VIEW_STATE_SHOW_SMS );
            }
        }
        catch( e )
        {
            EX_Log( "SMS_WidgetViewController::OnGetAccountInfoSucceeded( ) : " + e.message );
        }
    },

    OnLoginSucceeded:function( responseMessage )
    {
        try
        {
            this.ClearForceRegistrationView( );

            this.m_autoLoginView.handleGetAccountInfo( );
        }
        catch( e )
        {
            EX_Log( "SMS_WidgetViewController::OnLoginSucceeded( ) : " + e.message );
        }
    },

    OnVerificationSuccess : function( )
    {
        try
        {
            this.ChangeState( this.VIEW_STATE_SHOW_SMS );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'SMS_WidgetViewController::OnVerificationSuccess( )' );
        }
    },

    /**************************************************************************
      View update methods
     **************************************************************************/

    resetViews:function( )
    {
        this.m_registerView.reset( );
        this.m_autoLoginView.reset( );
        this.m_loginView.reset( );
        this.m_verificationView.reset( );
        this.m_smsView.reset( );
    },

    updateView:function( )
    {
        this.resetViews( );

        switch ( this.getState( ) )
        {
        case this.VIEW_STATE_SHOW_REGISTER:
            this.DrawRegisterView( );
            break;

        case this.VIEW_STATE_SHOW_AUTO_LOGIN:
            this.DrawAutoLoginView( );
            break;

        case this.VIEW_STATE_SHOW_LOGIN:
            this.DrawLoginView( );
            break;

        case this.VIEW_STATE_SHOW_VERIFICATION:
            this.DrawVerificationView( );
            break;

        case this.VIEW_STATE_SHOW_SMS:
            this.DrawSMSView( );
            break;

        default:
            this.DrawRegisterView( );
            break;
        }
    },

    DrawRegisterView:function( )
    {
        this.m_registerView.refresh( );
    },

    DrawAutoLoginView:function( )
    {
        this.m_autoLoginView.refresh( );
    },

    DrawLoginView:function( )
    {
        this.m_loginView.refresh( );
    },

    DrawVerificationView: function( )
    {
        this.m_verificationView.refresh( );
    },

    DrawSMSView: function( )
    {
        this.m_smsView.refresh( );
    }
};

// Need a container for function that may be called before
// VM_WidgetViewController exists

function VM_WidgetViewController_static()
{

/*  adjustHeight
        Intended to be overridden
        When an event occurs that requires more vertical screen screen real estate, call this.
        Returns true if height is adjusted
*/

    this.adjustHeight = function( )
    {
    return false;
    }
}

